#
# Copyright (c) 2016 Juniper Networks, Inc. All rights reserved.
#

import gevent
from gevent import monkey
monkey.patch_all()
import bottle
import base64
from bottle import GeventServer
from gevent.pool import Pool

def get_bottle_server(pool_size):
    """
    Custom gevent pool server for bottle
    """

    class GeventPoolServer(GeventServer):
        """ Gevent server with limited pool size
        """
        def __init__(self, host='127.0.0.1', port=8181, **options):
            super(GeventPoolServer, self ).__init__(host, port,
                spawn=Pool(size=pool_size), **options)

    return GeventPoolServer
# end get_bottle_server

# Open port for access to API server for trouble shooting
class LocalApp(object):

    def __init__(self, app, conf_info):
        self._http_host = 'localhost'
        self._http_port = conf_info['admin_port']
        self._http_app = bottle.Bottle()
        self._http_app.merge(app.routes)
        self._http_app.config.local_auth = True
        self._http_app.error_handler = app.error_handler
        self._conf_info = conf_info

        # 2 decorators below due to change in api between bottle 0.11.6
        # (which insists on global app) vs later (which need on specific
        # app)
        @self._http_app.hook('before_request')
        @bottle.hook('before_request')
        def local_auth_check(*args, **kwargs):
            if bottle.request.app != self._http_app:
                return
            # expect header to have something like 'Basic YWJjOmRlZg=='
            auth_hdr_val = bottle.request.environ.get('HTTP_AUTHORIZATION')
            if not auth_hdr_val:
                bottle.abort(401, 'HTTP_AUTHORIZATION header missing')
            try:
                auth_type, user_passwd = auth_hdr_val.split()
            except Exception as e:
                bottle.abort(401, 'Auth Exception: %s' %(str(e)))
            enc_user_passwd = auth_hdr_val.split()[1]
            user_passwd = base64.b64decode(enc_user_passwd)
            user, passwd = user_passwd.split(':')
            if (not self._conf_info.get('admin_user') == user or
                not self._conf_info.get('admin_password') == passwd):
                bottle.abort(401, 'Authentication check failed')

            # Add admin role to the request
            bottle.request.environ['HTTP_X_ROLE'] = 'admin'
    # end __init__

    def start_http_server(self):
        self._http_app.run(
            host=self._http_host, port=self._http_port,
            server=get_bottle_server(1024))
    # end start_http_server
# end class LocalApp
